local function FindSmallestPO2(num)
	return 2 ^ math.ceil(math.log(num)/math.log(2))
end

local function ScaleToPO2(xsize, ysize)
	if love.graphics.isSupported("npot") then return xsize, ysize end
	return FindSmallestPO2(xsize), FindSmallestPO2(ysize)
end

-- i've found xsize and ysize of 1/2 - 1/4 the screen resolution to be nice
-- smaller values make the bloom more apparent and give much better performance, but can make it look bad if too small
function CreateBloomEffect(xsize, ysize) 
	if not love.graphics.newPixelEffect
	or not love.graphics.isSupported
	or not love.graphics.isSupported("pixeleffect")
	or not love.graphics.isSupported("canvas") then
		return
	end
	
	local po2xsize, po2ysize = ScaleToPO2(xsize, ysize) -- scales x and y to next power of 2 if npot is false
		
	
	local xres, yres = love.graphics.getWidth(), love.graphics.getHeight()
	local po2xres, po2yres = ScaleToPO2(xres, yres)

	
	local quad = love.graphics.newQuad(0, 0, xsize, ysize, po2xsize, po2ysize)
	local scenequad = love.graphics.newQuad(0, 0, xres, yres, po2xres, po2yres)
	
	local canvas = {
		bloom = love.graphics.newCanvas(po2xsize, po2ysize),
		blur_horiz = love.graphics.newCanvas(po2xsize, po2ysize),
		blur_vert = love.graphics.newCanvas(po2xsize, po2ysize),
		scene = love.graphics.newCanvas(po2xres, po2yres),
	}
	
	local shaders = {
		blur_vert = love.graphics.newPixelEffect[[
			extern number rt_h = 256.0;
			
			extern number intensity = 1.0;

			number offset[3] = number[](0.0, 1.3846153846, 3.2307692308);
			number weight[3] = number[](0.2270270270, 0.3162162162, 0.0702702703);
			
			vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
			{
				vec4 texcolor = Texel(texture, texture_coords);
				vec3 tc = texcolor.rgb * weight[0];
				
				tc += Texel(texture, texture_coords + intensity * vec2(0.0, offset[1])/rt_h).rgb * weight[1];
				tc += Texel(texture, texture_coords - intensity * vec2(0.0, offset[1])/rt_h).rgb * weight[1];
				
				tc += Texel(texture, texture_coords + intensity * vec2(0.0, offset[2])/rt_h).rgb * weight[2];
				tc += Texel(texture, texture_coords - intensity * vec2(0.0, offset[2])/rt_h).rgb * weight[2];
				
				return color * vec4(tc, texcolor.a);
			}
		]],
		blur_horiz = love.graphics.newPixelEffect[[
			extern number rt_w = 256.0;
			
			extern number intensity = 1.0;

			number offset[3] = number[](0.0, 1.3846153846, 3.2307692308);
			number weight[3] = number[](0.2270270270, 0.3162162162, 0.0702702703);

			vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
			{
				vec4 texcolor = Texel(texture, texture_coords);
				vec3 tc = texcolor.rgb * weight[0];
				
				tc += Texel(texture, texture_coords + intensity * vec2(offset[1], 0.0)/rt_w).rgb * weight[1];
				tc += Texel(texture, texture_coords - intensity * vec2(offset[1], 0.0)/rt_w).rgb * weight[1];
				
				tc += Texel(texture, texture_coords + intensity * vec2(offset[2], 0.0)/rt_w).rgb * weight[2];
				tc += Texel(texture, texture_coords - intensity * vec2(offset[2], 0.0)/rt_w).rgb * weight[2];
				
				return color * vec4(tc, texcolor.a);
			}
		]],
		bloom = love.graphics.newPixelEffect[[
			extern number threshold = 1.0;
			
			number luminance(vec3 c)
			{
				return dot(c, vec3(0.3, 0.59, 0.11));
			}
			
			number highlights(vec3 c) {
				return smoothstep(threshold, 1.0, luminance(c));
			}
			
			vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
			{
				vec4 texcolor = Texel(texture, texture_coords);
				
				//return vec4(texcolor.rgb * highlights(texcolor.rgb), texcolor.a);
				
				return clamp((texcolor - threshold) / (1.0 - threshold), 0.0, 1.0);
				
				//vec4 fragcolor = vec4(0.0);
				//if (length(texcolor.rgb) >= threshold) fragcolor = texcolor;
				//return fragcolor;
			}
		]],
		
		combine = love.graphics.newPixelEffect[[
			extern Image bloomtex;
			
			extern number basesaturation = 1.0;
			extern number bloomsaturation = 1.0;
			
			extern number baseintensity = 1.0;
			extern number bloomintensity = 1.0;
			
			vec3 AdjustSaturation(vec3 color, number saturation)
			{
				// The constants 0.3, 0.59, and 0.11 are chosen because the
    			// human eye is more sensitive to green light, and less to blue.
    			vec3 grey = vec3(dot(color, vec3(0.3, 0.59, 0.11)));

    			return mix(grey, color, saturation);
			}
		
			vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords)
			{
				vec4 basecolor = Texel(texture, texture_coords);
				vec4 bloomcolor = Texel(bloomtex, texture_coords);
				
				bloomcolor.rgb = AdjustSaturation(bloomcolor.rgb, bloomsaturation) * bloomintensity;
				basecolor.rgb = AdjustSaturation(basecolor.rgb, basesaturation) * baseintensity;
				
				basecolor *= (1.0 - clamp(bloomcolor, 0.0, 1.0)), vec4(1.0);
				
				return vec4((basecolor + bloomcolor).rgb, 0.75);
			}
		]]
	}
	
	shaders.blur_horiz:send("rt_w", po2xsize)
	shaders.blur_vert:send("rt_h", po2ysize)

	
	local intensity_base, intensity_bloom = 1, 1
	local saturation_base, saturation_bloom = 1, 1
	local threshold_bloom = 0.5
	
	local debugdraw = false
	
	local bloom = {}
	
	function bloom:debugDraw(shoulddebugdraw)
		debugdraw = not not shoulddebugdraw
	end
	
	function bloom:setIntensity(ibase, ibloom)
		intensity_base = ibase
		intensity_bloom = ibloom
		
		shaders.combine:send("baseintensity", ibase)
		shaders.combine:send("bloomintensity", ibloom)
	end
	function bloom:getIntensity()
		return intensity_base, intensity_bloom
	end
	
	function bloom:setSaturation(sbase, sbloom)
		saturation_base = sbase
		saturation_bloom = sbloom
		
		shaders.combine:send("basesaturation", sbase)
		shaders.combine:send("bloomsaturation", sbloom)
	end
	function bloom:getSaturation()
		return saturation_base, saturation_bloom
	end
	
	function bloom:setThreshold(threshold)
		threshold_bloom = threshold
		
		shaders.bloom:send("threshold", threshold)
	end
	function bloom:getThreshold()
		return threshold_bloom
	end
	
	bloom:setIntensity(1, 1)
	bloom:setSaturation(1, 1)
	bloom:setThreshold(0.5) -- 0.25 is usually a nice threshold but I wanted 0.5 for my game
	
	-- call right before drawing the stuff you want bloomed
	function bloom:predraw()
		for k,v in pairs(canvas) do
			v:clear()
		end
		love.graphics.setCanvas(canvas.scene)
	end
	
	-- call after drawing the stuff you want bloomed
	function bloom:postdraw(func)	
		love.graphics.setCanvas()	
		love.graphics.setColor(255, 255, 255)
		love.graphics.setBlendMode("alpha")
		
		love.graphics.push()
		love.graphics.scale(po2xsize/po2xres, po2ysize/po2yres)
		
		-- apply bloom extract shader
		love.graphics.setCanvas(canvas.bloom)
		love.graphics.setPixelEffect(shaders.bloom)
		love.graphics.drawq(canvas.scene, scenequad, 0, 0)
		
		love.graphics.pop()
		
		love.graphics.setBlendMode("premultiplied")
		
		-- apply horizontal blur shader to extracted bloom
		love.graphics.setCanvas(canvas.blur_horiz)
		love.graphics.setPixelEffect(shaders.blur_horiz)
		love.graphics.drawq(canvas.bloom, quad, 0, 0)
		
		-- apply vertical blur shader to blurred bloom
		love.graphics.setCanvas(canvas.blur_vert)
		love.graphics.setPixelEffect(shaders.blur_vert)
		love.graphics.drawq(canvas.blur_horiz, quad, 0, 0)
		
		if func == nil then
			love.graphics.setCanvas()
		else
			func()
		end
		
		-- render final scene combined with bloom canvas
		shaders.combine:send("bloomtex", canvas.blur_vert)
		love.graphics.setPixelEffect(shaders.combine)
		love.graphics.drawq(canvas.scene, scenequad, 0, 0)
		
		love.graphics.setPixelEffect()
		
		love.graphics.setBlendMode("alpha")
		
		if debugdraw then
			love.graphics.draw(canvas.bloom, 0, 0)
			love.graphics.draw(canvas.blur_horiz, po2xsize, 0)
			love.graphics.draw(canvas.blur_vert, po2xsize*2, 0)
		end
	end
	
	return bloom
end
